/*
 * A file accessor employing a "host" file provided via the C library.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "host_file_accessor.h"

HostFileAccessor::HostFileAccessor(FILE *fp, fileid_t fileid)
: Accessor(fileid), _fp(fp)
{
    /* Initialise the size of the file. */

    struct stat buf;

    if (!fstat(fileno(fp), &buf))
        _size = buf.st_size;
    else
        _size = 0;
}

/* Return the size of the file. */

offset_t HostFileAccessor::get_size()
{
    return _size;
}

/* Update the size of the file. */

void HostFileAccessor::set_size(offset_t size)
{
    _size = size;
}

/* Perform any closing operation on the file. */

void HostFileAccessor::close()
{
    fclose(_fp);
}

/* Data transfer helper methods. */

void HostFileAccessor::fill_populated(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;

    /* Seek to the offset in the file. */

    fseek(_fp, filepos, SEEK_SET);

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    size_t nread = fread((void *) addr, sizeof(char), flexpage->size, _fp);

    /* Pad the flexpage with zero. */

    memset((void *) (addr + nread), 0, flexpage->size - nread);
}

void HostFileAccessor::flush_populated(Flexpage *flexpage)
{
    flexpage->region->flush();
}

// vim: tabstop=4 expandtab shiftwidth=4
